// Create the charts when the web page loads
window.addEventListener('load', onload);

function onload(){
  chartT = createTemperatureChart();
  chartH = createHumidityChart();
  chartC = createCO2Chart();
  chartS = createSoundChart();
  chartL = createLightChart();
  chartG = createGlobalChart();
}

// Create Global Chart
function createGlobalChart() {
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-global',
      type: 'spline' 
    },
    series: [
      {
        name: 'global'
      }
    ],
    title: { 
      text: undefined
    },
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: '/5' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}

// Create Temperature Chart
function createTemperatureChart() {
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-temperature',
      type: 'spline' 
    },
    series: [
      {
        name: 'ssenseHDC2010'
      }
    ],
    title: { 
      text: undefined
    },
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
      series: { 
        color: '#21059e' 
      }
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: 'Temperature Celsius Degrees' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}

// Create Humidity Chart
function createHumidityChart(){
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-humidity',
      type: 'spline'  
    },
    series: [{
      name: 'ssenseHDC2010'
    }],
    title: { 
      text: undefined
    },    
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
      series: { 
        color: '#00add6' 
      }
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: 'Humidity (%)' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}

// Create CO2 Chart
function createCO2Chart() {
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-co2',
      type: 'spline'  
    },
    series: [{
      name: 'ssenseCCS811'
    }],
    title: { 
      text: undefined
    },    
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
      series: { 
        color: '#7e7e7e' 
      }
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: 'CO2 (ppm)' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}


// Create Sound Chart
function createSoundChart(){
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-sound',
      type: 'spline'  
    },
    series: [{
      name: 'Dabble-Soundmeter'
    }],
    title: { 
      text: undefined
    },    
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
      series: { 
        color: '#a61616' 
      }
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: 'Sound (dB)' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}


// Create Sound Chart
function createLightChart(){
  var chart = new Highcharts.Chart({
    accessibility: {
      enabled: false
  },
    chart:{ 
      renderTo:'chart-light',
      type: 'spline'  
    },
    series: [{
      name: 'BH1750'
    }],
    title: { 
      text: undefined
    },    
    plotOptions: {
      line: { 
        animation: false,
        dataLabels: { 
          enabled: true 
        }
      },
      series: { 
        color: '#dcd500f8' 
      }
    },
    xAxis: {
      type: 'datetime',
      dateTimeLabelFormats: { second: '%H:%M:%S' }
    },
    yAxis: {
      title: { 
        text: 'Light (lux)' 
      }
    },
    credits: { 
      enabled: false 
    }
  });
  return chart;
}


Highcharts.setOptions({
  time: {
      useUTC: false //display the local time
  }
});