

//get the average of values in an array
function avg(values) {
  let sum = 0;
  values.forEach((value) => {
    sum += value;
  });
  return sum / values.length;
}

// convert time to human-readable format YYYY/MM/DD HH:MM:SS
function timestampToDateTime(timestamp) {
  var epochDate = new Date(Number(timestamp));
  var dateTime = epochDate.getFullYear() + "/" +
    ("00" + (epochDate.getMonth() + 1)).slice(-2) + "/" +
    ("00" + epochDate.getDate()).slice(-2) + " " +
    ("00" + epochDate.getHours()).slice(-2) + ":" +
    ("00" + epochDate.getMinutes()).slice(-2) + ":" +
    ("00" + epochDate.getSeconds()).slice(-2);

  return dateTime;
}

// function to plot values on charts
function plotValues(chart, timestamp, value) {
  var x = new Date(Number(timestamp)).getTime();
  var y = Number(value);
  if (chart.series[0].data.length > 40) {
    chart.series[0].addPoint([x, y], true, true, true);
  } else {
    chart.series[0].addPoint([x, y], true, false, true);
  }
}

// DOM elements
const loginElement = document.querySelector('#login-form');
const contentElement = document.querySelector("#content-sign-in");
const userDetailsElement = document.querySelector('#user-details');
const authBarElement = document.querySelector('#authentication-bar');
const loadDataButtonElement = document.getElementById('load-data');
const chartsCheckboxElement = document.querySelector('input[name=charts-checkbox]');

// DOM elements for sensor readings
const cardsReadingsElement = document.querySelector("#cards-div");
const chartsDivElement = document.querySelector('#charts-div');
const updateElement = document.getElementById("lastUpdate")


const recordButton = document.getElementById("record-button");
const recordStatusLabel = document.getElementById("record-status-label");

const recordsSelect = document.getElementById('records-select');

const recordsSelectDiv = document.getElementById('record-select-div');

const recordStatusText = document.getElementById('record-status-text');


//averages
const humidityAvg = document.getElementById("hum-avg");
const soundAvg = document.getElementById("sound-avg");
const temperatureAvg = document.getElementById("temp-avg");
const co2Avg = document.getElementById("co2-avg");
const lightAvg = document.getElementById("light-avg");
const globalAvg = document.getElementById("global-grade");

//stars
//averages
const globalStar0 = document.getElementById("star-0");
const globalStar1 = document.getElementById("star-1");
const globalStar2 = document.getElementById("star-2");
const globalStar3 = document.getElementById("star-3");
const globalStar4 = document.getElementById("star-4");

const globalStars = [globalStar0, globalStar1, globalStar2, globalStar3, globalStar4];

var recordId = null;
var recordingInProgress = true;

var dbPath;
var dbRef;
var recordsRef;
var chartRef;
var recordStatusRef;
var recInProgRef;

var liveRecordsRef;

//INIT FUNCTIONS

function initDbRefs(uid) {
  dbPath = 'UsersData/' + uid.toString();
  // Database paths (with user UID)
  var recordsPath = dbPath + '/records';
  // Database references
  dbRef = firebase.database().ref(dbPath);
  recordsRef = firebase.database().ref(recordsPath);
  recordStatusRef = firebase.database().ref(dbPath + '/recordStatus');
  recInProgRef = firebase.database().ref(dbPath + '/recordStatus/recordingInProgress');
}

function initElements() {
  populateRecordsSelect();
  listenToRecordingChange();

  //change record status when confirmed
  recordStatusRef.child('/confirmed').on('value', (snapshot) => {
    let confirmed = snapshot.val(); //confirmed timestamp or -1
    if (recordingInProgress && confirmed != -1) {
      recordsRef.child('/' + confirmed).once('value', (snapshot) => {
        recordStatusText.textContent = 'Record in progress (started on ' + timestampToDateTime(confirmed) + ")";
      });
      setLiveDisplay(confirmed);
    }
  });
}


function listenToRecordingChange() {
  recInProgRef.on('value', (snapshot) => {
    recordingInProgress = snapshot.val();
    updateRecordStatus();
  });
}

function invertRecordStatus() {
  recInProgRef.off();
  recInProgRef.set(!recordingInProgress)
    .then(() => {
      recordingInProgress = !recordingInProgress;
      if (!recordingInProgress) {
        recordStatusRef.child('/confirmed').set(-1);
      }
      updateRecordStatus();
    })
    .finally(() => {
      listenToRecordingChange();
    });
}

function updateRecordStatus() {
  if (recordingInProgress) {
    recordButton.setAttribute("class", "stop-button");
    recordButton.textContent = "STOP RECORDING";
    recordsSelectDiv.style.display = 'none';
    recordStatusLabel.removeAttribute("hidden");
    recordStatusText.textContent = "Request for recording";
    resetData();
  } else {
    recordButton.setAttribute("class", "start-button");
    recordButton.textContent = "START RECORDING";
    populateRecordsSelect();
    recordsSelectDiv.style.display = "inline-block";
    recordStatusLabel.setAttribute("hidden", true);
  };
  //display button
  recordButton.removeAttribute('hidden');
}

function setLiveDisplay(id) {
  liveRecordsRef = recordsRef.child('/' + id);
  liveRecordsRef.on('value', (snapshot) => {
    computeAndDisplayData(snapshot.val());
  });
}


// Function to handle the change event of the records select
function handleRecordSelectChange(event) {
  recordId = event.target.value;
  updateDisplayedData();
}

function renewCharts() {
  chartT.destroy();
  chartH.destroy();
  chartC.destroy();
  chartS.destroy();
  chartL.destroy();
  chartG.destroy();
  // Render new charts to display new range of data
  chartT = createTemperatureChart();
  chartH = createHumidityChart();
  chartC = createCO2Chart();
  chartS = createSoundChart();
  chartL = createLightChart();
  chartG = createGlobalChart();
}

function displayGlobalStars(gavg) {
  for (let i = 0; i < globalStars.length; i++) {
    if (gavg * 5 - (i + 1) >= 0) {
      console.log(globalStars[i].class);
      globalStars[i].setAttribute("class", "fas fa-star");
    } else {
      if (gavg * 10 - (i) * 2 > 0) {
        globalStars[i].setAttribute("class", "fas fa-star-half-alt"); //half star
      } else {
        globalStars[i].setAttribute("class", "far fa-star");
      }
    }
  }

}


function computeAndDisplayData(records) {
  if (records) {
    if (!recordingInProgress) {
      renewCharts();
    }
    let globals = [];
    let humidities = [];
    let co2Values = [];
    let temperatures = [];
    let soundValues = [];
    let lights = [];
    let i;
    for (i = 0; i < records.size - 1; i++) {
      //some indices may be missing
      try {
        humidities.push(parseFloat(records[i].humidity));
        co2Values.push(parseFloat(records[i].co2));
        temperatures.push(parseFloat(records[i].temperature));
        soundValues.push(parseFloat(records[i].sound));
        lights.push(parseFloat(records[i].light));
        globals.push(parseFloat(records[i].global));
        plotValues(chartT, records[i].timestamp, records[i].temperature);
        plotValues(chartH, records[i].timestamp, records[i].humidity);
        plotValues(chartC, records[i].timestamp, records[i].co2);
        plotValues(chartS, records[i].timestamp, records[i].sound);
        plotValues(chartL, records[i].timestamp, records[i].light);
        plotValues(chartG, records[i].timestamp, records[i].global*5);
      } catch (error) {
      }

    }

    //if all fields are present
    if (humidities.length * co2Values.length * temperatures.length * soundValues.length * lights.length != 0) {
      humidityAvg.textContent = avg(humidities).toFixed(2);
      co2Avg.textContent = avg(co2Values).toFixed(2);
      temperatureAvg.textContent = avg(temperatures).toFixed(2);
      soundAvg.textContent = avg(soundValues).toFixed(2);
      lightAvg.textContent = avg(lights).toFixed(2);
      let gavg = avg(globals);
      globalAvg.textContent = (gavg*5).toFixed(2);
      displayGlobalStars(gavg);
    }


  } else {
    //no record found for this id
  }
}

function resetData() {
  humidityAvg.textContent = "--"
  co2Avg.textContent = "--"
  temperatureAvg.textContent = "--"
  soundAvg.textContent = "--"
  lightAvg.textContent = "--"
  globalAvg.textContent ="--";
  for (let i =0; i<globalStars.length; i++){
    globalStars[i].setAttribute("class", "far fa-star");
  }
  renewCharts();
}
function updateDisplayedData() {
  recordsRef.child(recordId).once('value', (snapshot) => {
    computeAndDisplayData(snapshot.val());
  });
}

// Function to populate the select element with first and last IDs for each reading
function populateRecordsSelect() {
  recordsSelect.textContent = "";
  recordsRef.on('value', (snapshot) => {
    const recordsSnapshot = snapshot.val();

    const recordsKeys = Object.keys(recordsSnapshot);

    for (let i = recordsSnapshot.number - 1; i >= 0; i--) {
      const readingId = recordsKeys[i];

      const recordsData = recordsSnapshot[readingId];
      const recordsArray = Object.keys(recordsData);

      if (recordsArray.length > 0 && recordsData[recordsSnapshot[readingId].size - 1] != undefined) {
        const firstTimestamp = recordsData[recordsArray[0]].timestamp;
        const lastTimestamp = recordsData[recordsSnapshot[readingId].size - 1].timestamp;


        // Create an option element for each reading and its first and last timestamps
        const option = document.createElement('option');
        option.value = readingId; // Set the value as the ID of the reading
        option.textContent = `${readingId}: [${timestampToDateTime(firstTimestamp)} - ${timestampToDateTime(lastTimestamp)}]`;
        recordsSelect.appendChild(option);
      }
    }
    if (recordsKeys.length != 0) {
      //select last record by default
      recordId = recordsKeys[recordsSnapshot.number - 1];
      updateDisplayedData();
    }
  });
}

// MANAGE LOGIN/LOGOUT UI
const setupUI = (user) => {
  if (user) {
    //toggle UI elements
    loginElement.style.display = 'none';
    contentElement.style.display = 'block';
    authBarElement.style.display = 'block';
    userDetailsElement.style.display = 'block';
    userDetailsElement.innerHTML = user.email;

    // get user UID to get data from database
    var uid = user.uid;

    initDbRefs(uid);
    initElements();

    // IF USER IS LOGGED OUT
  } else {
    // toggle UI elements
    loginElement.style.display = 'block';
    authBarElement.style.display = 'none';
    userDetailsElement.style.display = 'none';
    contentElement.style.display = 'none';
  }
}
